/*******************************************************************************
This do file is to produce textbook figures in Undergraduate Econometrics 
textbook of Professor Diebold. 

Author: Theodore Caputi, Donato Onorato, JoonYup Park, Sarah Winton
Date: 8/23/2017
Project: Undergraduate Econometrics Textbook (Liquors)
Contact: joonyup@wharton.upenn.edu

Instructions: 
	1. Change the working directory "userdir" in the prelim before runnin the script.
	
	2. If you wish to save the graphical output, please refer to the Stata code for 
		Wages data. You may use the command "graph export".

Note: 

	-Stata does not have an inherent command to perform Chow Test. You may calculate
	the F-stat by manually setting the cut-off and calculating various statistics using
	fitted values and residuals. 

Updates:
	Finalized 8/25/2017
	
********************************************************************************/

clear all
set more off
set graphics off //comment this out if you want to see the graph as you go


*** Set working directory
global userdir "" // <- input your working directory here
global logs "${userdir}/logs"
global tabfig "${userdir}/tabfig"


*** Input file
global liquordata "${userdir}/DataLiquor.csv"


*** Output file

/* Start logging */
//cap log close
//log using "${logs}\CodeStataWages_`c(current_date)'.txt", replace text
//Use the log if you wish to keep track of your progress


*Import liquor dataset and declare it to be time-series data
insheet using "${liquordata}", clear
rename liquorsales1987m12014m12 liquor
gen time = tm(1987m1) + _n - 1
format time %tm
tsset time


*Liquor Sales 
graph twoway tsline liquor, xtitle("Time") ytitle("Liquor Sales") ///
	name(ts_liquor, replace)
	
*Log Liquor Sales	
gen ln_liquor = log(liquor)
label var ln_liquor "Log(LiquorSales)"
graph twoway tsline ln_liquor, xtitle("Time") ytitle("Log Liquor Sales") ///
	name(ts_lnliquor, replace)

	
*Linear Trend Estimation
gen time_index = _n
reg ln_liquor time_index

*Residual Plot and Linear Trend Estimation
predict ln_ls_pred
predict ln_ls_resid, resid 
label var ln_ls_pred "Fitted Liquor Sales"
label var ln_ls_resid "Residuals"

graph twoway ///
	tsline ln_ls_pred || ///
	tsline ln_liquor || ///
	tsline ln_ls_resid, xtitle("Time") 
	
*Create monthly dummies
gen month = month(dofm(time))
tab month, gen(month)
drop month

*Linear Trend Estimation with Seasonal Dummies
reg ln_liquor time_index month*, nocons

*Residual Plot, Linear Trend Estimation with Seasonal Dummies 
predict ln_ls_seas_pred
predict ln_ls_seas_resid, resid
label var ln_ls_seas_pred "Fitted Liquor Sales"
label var ln_ls_seas_resid "Residuals"

graph twoway ///
	tsline ln_ls_seas_pred || ///
	tsline ln_liquor || ///
	tsline ln_ls_seas_resid, xtitle("Time") 


*Seasonal Pattern 
reg ln_liquor time_index month*, nocons

mat beta = e(b)' //extract regression coefficients
svmat beta, names(col)
rename y1 seas_coef
gen month = _n - 1 in 2/13

graph twoway ///
	line seas_coef month if !missing(month), ///
	xlab(1(1)12) title("Estimated Seasonal Factors") xtitle("Month") ytitle("Factor")

drop month seas_coef

	
*Log-Quadratic Trend Estimation
gen time_index_sq = time_index^2
reg ln_liquor time_index time_index_sq


*Residual Plot, Log-Quadratic Trend Estimation
predict ln_quad_ls_pred
predict ln_quad_ls_resid, resid
label var ln_quad_ls_pred "Fitted Liquor Sales"
label var ln_quad_ls_resid "Residuals"

graph twoway ///
	tsline ln_quad_ls_pred || ///
	tsline ln_liquor || ///
	tsline ln_quad_ls_resid, xtitle("Time") 


*Log-Quadratic Trend Estimation with Seasonal Dummies
reg ln_liquor time_index time_index_sq month*, nocons


*Residual Plot, Log-Quadratic Trend Estimation with Seasonal Dummies
predict ln_quad_ls_seas_pred
predict ln_quad_ls_seas_resid, resid
label var ln_quad_ls_seas_pred "Fitted Liquor Sales"
label var ln_quad_ls_seas_resid "Residuals"

graph twoway ///
	tsline ln_quad_ls_seas_pred || ///
	tsline ln_liquor || ///
	tsline ln_quad_ls_seas_resid, xtitle("Time") 

	
*Expanding Window Regression 

**Define parameters
qui count 
local n = `r(N)'
local k = 121 //We will start from Jan. 1997. 121th row of the data refers to Jan. 1997

gen coef_expand=.
qui forval i = `k'/`n' {

	*Regress log liquor on trend and seasonal dummies and extract regression coefficient
	reg ln_liquor time_index month* if time_index <= `i', nocons
	mat b = e(b)
	local b_expand = b[1,1]
	
	replace coef_expand = `b_expand' if time_index == `i'
}
	
graph twoway ///
	tsline coef_expand if !missing(coef_expand), ///
	title("Expanding Window Regression") xtitle("Window Cut-off Time") ytitle("Coefficient of Trend") xlabel(, angle(vertical))

drop coef_expand
	
	
*Rolling Window Regression	

**Define parameters
qui count
local n = `r(N)'
local w = 120 //We fix the window length to be 10 years
local loopstart = 1
local loopend = `n'-`w'+1

gen coef_rolling=.
qui forval i = `loopstart'/`loopend' {

	*Define window cut-off
	local w_start = `i'
	local w_end = `i'+`w'-1
	
	*Regress log liquor on trend and seasonal dummies and extract regression coefficient
	reg ln_liquor time_index month* if time_index >= `w_start' & time_index <= `w_end', nocons
	mat b = e(b)
	local b_roll = b[1,1]
	
	replace coef_rolling = `b_roll' if time_index == `i'
}

graph twoway ///
	tsline coef_rolling if !missing(coef_rolling), ///
	title("Rolling Window Regression") xtitle("Window Cut-off Start Time") ytitle("Coefficient of Trend") xlabel(, angle(vertical)) note("Window Length: `w' months")
	

*Scatter Plot of e_t against e_t-1

**Create lagged residuals
gen ln_quad_ls_seas_resid_lag1 = ln_quad_ls_seas_resid[_n-1]

graph twoway ///
	scatter ln_quad_ls_seas_resid ln_quad_ls_seas_resid_lag1, xtitle("Residual Lag1") ytitle("Residual")
	

*Durbin-Watson Test for AR(1) Distrubance
reg ln_liquor time_index time_index_sq month*, nocons	
estat dwatson
	

*Regress on trend + trend^2 + season + p-lags	

**Define parameter 
local P = 4

**Create lagged log(liquor sales)
forval i = 1/`P' {

	gen ln_liquor_l`i' = ln_liquor[_n-`i']
}

reg ln_liquor time_index time_index_sq month* ln_liquor_*, nocons
estat ic

*Residual Plot, Log-Quadratic Trend Estimation with Seasonal Dummies with P-lags
predict ln_quad_ls_seas_lag_pred
predict ln_quad_ls_seas_lag_resid, resid
label var ln_quad_ls_seas_lag_pred "Fitted Liquor Sales"
label var ln_quad_ls_seas_lag_resid "Residuals"

graph twoway ///
	tsline ln_quad_ls_seas_lag_pred || ///
	tsline ln_liquor || ///
	tsline ln_quad_ls_seas_lag_resid, xtitle("Time") 


*Correlogram of residuals (Auto-correlation and Partial Auto-correlation)
corrgram ln_quad_ls_seas_resid


cap log close
